//////////////////////////////////////////////////////////////////////
//
//  PREVIEW.PRG
//
//  Copyright:
//      Alaska Software, (c) 1997-2006. Alle Rechte vorbehalten.
//
//  Inhalt:
//      Das Programm enthlt die Klasse XbpPreview, welche die Ausgabe
//      auf einem Drucker WYSIWYG als Vorschau in einem Fenster anzeigt.
//      Die Anzeige kann gezoomt werden und lt sich scrollen
//
//  Bemerkungen:
//      Die Prozedur Main erzeugt ein XbpDialog Fenster in das XbpPreview
//      sowie Pushbuttons zum Zoomen und Drucken eingebettet sind.
//      Die Anzeige der Druckausgabe erfolgt in der Funktion DrawDB(),
//      welche ein einfaches Listing einer Datenbank erzeugt.
//
//////////////////////////////////////////////////////////////////////

#include "Appevent.ch"
#include "Common.ch"
#include "Font.ch"
#include "Gra.ch"
#include "Xbp.ch"


#IF XPPVER > 1900331 .AND. XPPVER < 2000000
   // Xbase++ 1.9 compatibility
   #pragma library( "xppui3.lib" )

   #ifndef XBP_RELXPOS
   #define XBP_RELXPOS                    0x10
   #define XBP_RELYPOS                    0x20
   #define XBP_RELWIDTH                   0x40
   #define XBP_RELHEIGHT                  0x80
   #define XBP_XCENTER                    0x1000
   #define XBP_YCENTER                    0x2000
   #endif
#endif

PROCEDURE AppSys()
// Wird hier nicht bentigt
RETURN

PROCEDURE Main
   LOCAL nEvent, mp1, mp2
   LOCAL oDlg, oXbp, drawingArea, oView

// Xbase++ 1.9 compatibility
#IF XPPVER > 1900331 .AND. XPPVER < 2000000
   LayoutManager()
#ENDIF

*   SET DEFAULT TO "..\..\..\DATA"
*   USE Cars NEW
   USE d:\ALASKA\DATA\CARS.DBF

   oDlg := XbpDialog():new( SetAppWindow(), , {10,10}, {400,410}, , .F.)
   oDlg:border    := XBPDLG_RAISEDBORDERTHICK_FIXED
   oDlg:title     := "Vorschau-Dialog"
*  oDlg:minButton := .F.
*  oDlg:maxButton := .F.
   oDlg:taskList  := .T.
   oDlg:drawingArea:resize := {| aOldSize, aNewSize, oSelf | MainResize(aOldSize, aNewSize, oSelf) }
   oDlg:create()

   drawingArea := oDlg:drawingArea
   drawingArea:setFontCompoundName( "8.Helv" )

   oView := XbpPreview():new( drawingArea , , { 0, 0}, {270,380} )
   oView:drawBlock := {|oPS, nPageNo, nNumRows| FormatPage(oPS, nPageNo, @nNumRows) }
*  oView:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
   oView:create()

   oXbp := XbpPushButton():new( drawingArea, , {283,325}, {100,30} )
   oXbp:caption := "Zoom ++"
   oXbp:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
   oXbp:create()
   oXbp:activate := {|| oView:zoom( oView:zoomFactor + 0.25 ) }

   oXbp := XbpPushButton():new( drawingArea, , {283,275}, {100,30} )
   oXbp:caption := "Zoom - -"
   oXbp:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
   oXbp:create()
   oXbp:activate := {|| oView:zoom( oView:zoomFactor - 0.25 ) }

   oXbp := XbpPushButton():new( drawingArea, , {283,223}, {100,30} )
   oXbp:caption := "Drucken"
   oXbp:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
   oXbp:create()
   oXbp:activate := {|| oView:print() }

   oXbp := XbpPushButton():new( drawingArea, , {283,171}, {100,30} )
   oXbp:caption := "OK"
   oXbp:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
   oXbp:create()
   oXbp:activate := {|| PostAppEvent( xbeP_Close ) }

   oDlg:show()
   SetAppFocus( oXbp )

   nEvent := 0
   DO WHILE nEvent <> xbeP_Close
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO
RETURN



/*
 * Funkion zum Formatieren einer Seite fr die Vorschau. Diese
 * wird ber den ":drawBlock" Codeblock aufgerufen und bekommt
 * drei Parameter [bergeben. "oPS" ist der Presentation Space
 * in den gezeichnet werden soll. Parameter "nPageNo" bestimmt
 * die Seite, die fr den Preview formatiert werden soll.
 * "nNumRows" gibt die Anzahl der Zeilen pro Seite an und wird
 * zur Positionierung des Datenbank-Cursors fr Seiten ungleich
 * der ersten Seite bentigt. Fr die erste Seite ("nPageNo"
 * == 1) mu diese von der Funktion berechnet und zurckgegeben
 * werden. Die Rekordnummer, ab der aus der Datenbank gelesen
 * werden soll, ergibt sich fr Seitennummern grer 1 aus
 *
 *   nRecNo := ((nPageNo -1) * nNumRows) +1
 *
 * Rckgabewert ist ein grafisches Segment mit den grafischen
 * Primitiven, die zum Anzeigen der Seite ntig sind.
 *
 * Hinweis zur Seitenformatierung und Anzeige:
 *   Die grafischen Primitiven zum Zeichnen der einzelnen
 *   Seiten werden jeweils in einem grafischen Segment gespei-
 *   chert. Diese werden jeweils [ber die Evaluierung des
 *   ":drawBlock" Codeblocks angefordert. Die Segmente werden
 *   dann entweder im Preview-Fenster zum Anzeigen der Vorschau
 *   oder auf dem Drucker abgespielt, um die entsprechende Seite
 *   darzustellen
 */
FUNCTION FormatPage( oPS, nPageNo, nNumRows )
   LOCAL i, imax   := FCount()
   LOCAL aFields   := DbStruct()
   LOCAL aPageSize := oPS:setPageSize()[1]
   LOCAL aPosX[imax], nY, xValue
   LOCAL nFontWidth, nFontHeight
   LOCAL nSegment

   // Maximale Hhe und Breite eines Buchstabens
   // im aktuellen Font feststellen
   nFontWidth  := oPS:setFont():width
   nFontHeight := oPS:setFont():height

   aPosX[1]    := nFontWidth/2
   FOR i:=2 TO imax
      aPosX[i] := aPosX[i-1] + nFontWidth/2 + ;
                  Max( Len(aFields[i-1,1]),aFields[i-1,3] ) * nFontWidth
   NEXT

   IF nPageNo == 1
      GO TOP
   ELSE
      GO ((nPageNo -1) * nNumRows) +1
   ENDIF

   // Anzeige der grafischen Primitiven whrend der Berechnung
   // der Seiten unterdrcken
   GraSegDrawMode( oPS, GRA_DM_RETAIN )

   //
   // Berechnung der Seite
   //
   // ffne neues Segment
   nSegment := GraSegOpen( oPS )

   // Feldnamen und Spaltentrennlinien zeichnen
   nY := aPageSize[2] - 2 * nFontHeight
   FOR i:=1 TO imax
      GraStringAt( oPS, { aPosX[i], nY }, FieldName(i) )
      GraLine( oPS, {aPosX[i]-nFontWidth/4, 0}, {aPosX[i]-nFontWidth/4, aPageSize[2]} )
   NEXT

   // Trennlinie fr Kopfzeile zeichnen
   nY -= nFontHeight/2
   GraLine( oPS, {0, nY}, {aPageSize[1], nY} )

   // Seite mit Daten vollschreiben
   DO WHILE nY >= ( nFontHeight + 2 )
      nY -= ( nFontHeight + 2 )

      FOR i:=1 TO imax
         xValue := FieldGet(i)
         DO CASE
            CASE aFields[i,2] == "D"
               xValue := DtoC( xValue )
            CASE aFields[i,2] == "L"
               xValue := IIf( xValue, "T", "F" )
            CASE aFields[i,2] == "M"
               xValue := Left( xValue, 10 )
            CASE aFields[i,2] == "N"
               xValue := Str( xValue )
         ENDCASE

         GraStringAt( oPS, { aPosX[i], nY }, xValue )
      NEXT

      SKIP
      IF Eof()
         EXIT
      ENDIF
   ENDDO

   IF nPageNo == 1
      nNumRows := RecNo() -1
   ENDIF

   // Schliee Segment und zurcksetzen des
   // Segment Drawing Mode
   GraSegClose( oPS )

   GraSegDrawMode( oPS, GRA_DM_DRAWANDRETAIN )

RETURN nSegment



/*
 * Klassendeklaration
 */
CLASS XbpPreview FROM XbpStatic
   PROTECTED:
   VAR oView                      // XbpDialog als Anzeigebereich
   VAR oSquare                    // XbpStatic zum Fllen der Lcke zw. Scrollbars
   VAR oPresSpace                 // XbpPresSpace
   VAR oFont                      // XbpFont
   VAR aViewPort                  // Viewport zum zoomen
   VAR aPageSize                  // Seitengre zum Drucken
   VAR aCharAttr                  // Attribute fr Strings (GRA_AS_BOX)
   VAR aAreaAttr                  // Attribute fr Flchen (GRA_AA_COLOR)
   VAR nPagePos                   // akt. Seitenposition
   VAR nRows                      // Anz. der Zeilen pro Seite

   METHOD setOrigin               // Ursprung fr Viewport setzen
   METHOD calcScrollBox           // Gre der ScrollBox bei Scrollbars berechnen

   METHOD startDoc                // Drucken beginnen
   METHOD endDoc                  // Drucken beenden

   EXPORTED:
   VAR hScroll    READONLY        // Horizontaler Scrollbar
   VAR vScroll    READONLY        // Vertikaler Scrollbar
   VAR oPagePos                   // Static zum Anzeigen der akt. Seitenposition
   VAR oNextPage                  // Button zum Anzeigen der naechsten Seite
   VAR oPrevPage                  // Button zum Anzeigen der vorigen Seite
                                  ** Konfiguration
   VAR drawBlock                  // Codeblock ruft Func zum Zeichnen auf
   VAR printer                    // XbpPrinter
   VAR zoomFactor                 // Zoomfaktor von 1 bis n
   VAR segments                   // Array mit Segment IDs
                                  ** Lebenszyklus
   METHOD init                    //
   METHOD create                  //
   METHOD destroy                 //
                                  ** Anzeige
   METHOD setSize                 // Gre definieren
   METHOD hScroll                 // Horizontal scrollen
   METHOD vScroll                 // Vertical scrollen
   METHOD zoom                    // Zoomen
   METHOD presSpace               // Pesentation Space abfragen
   METHOD setViewPort             // Gre vom Viewport einstellen
   METHOD drawPage                // Zeichnen einer Seite
                                  ** Drucken
   METHOD print                   // Ausgeben der Seiten auf dem Drucker

   METHOD numPages                // Ermitteln der Anzahl der Seiten
   METHOD pageNo                  // Ermitteln der Nummer der akt. Seite

   METHOD drawNextPage            // Zeichnen der nchsten Seite
   METHOD drawPrevPage            // Zeichnen der vorigen Seite
ENDCLASS



/*
 * Objekt initialisieren
 */
METHOD XbpPreview:init( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::XbpStatic:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   ::xbpStatic:type  := XBPSTATIC_TYPE_RAISEDRECT

   ::oView           := XbpDialog():new( self )

   ::vScroll         := XbpScrollbar():new( self )
   ::vScroll:type    := XBPSCROLL_VERTICAL
   ::vScroll:cargo   := 0
   ::vScroll:autoTrack := .F.

   ::oPagePos        := XbpPushButton():new( self )
   ::oPagePos:activate := { || MsgBox("Springe zu Seite n (nicht implementiert)") }

   ::oNextPage       := XbpPushButton():new( self )
   ::oNextPage:caption := "+"
   ::oNextPage:activate := { || ::drawNextPage() }
   ::oPrevPage       := XbpPushButton():new( self )
   ::oPrevPage:caption := "-"
   ::oPrevPage:activate := { || ::drawPrevPage() }

   ::hScroll         := XbpScrollbar():new( self )
   ::hScroll:type    := XBPSCROLL_HORIZONTAL
   ::hScroll:cargo   := 0
   ::hScroll:autoTrack := .F.

   ::oSquare         := XbpStatic():new( self )
   ::oSquare:type    := XBPSTATIC_TYPE_RECESSEDBOX

   // Initialisieren des Presentation Space fr die
   // Ausgabe der Vorschau. Um genaue Textausgabe
   // sicherzustellen, wird hier explizit der Modus
   // "hohe Textprzision" eingestellt
   ::oPresSpace      := XbpPresSpace():new()
   ::oPresSpace:mode := XBPPS_MODE_HIGH_PRECISION

   ::zoomFactor      := 1
   ::segments        := {}
   ::nPagePos        := 1
   ::nRows           := 0

   // Standard Drucker bereitstellen
   ::printer         := XbpPrinter():new()

    // Beende Applikation wenn kein Drucker installiert ist.
   IF ::printer:list() == NIL
      MsgBox( "Fehler - Es sind keine Drucker installiert!", "Preview Beispiel" )
      QUIT
   ENDIF

*   ::xbpStatic:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::oView    :layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::vScroll  :layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::oPagePos :layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::oNextPage:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::oPrevPage:layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::hScroll  :layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS
*   ::oSquare  :layoutAlign := XBP_LEFTEDGE+XBP_TOPEDGE+XBP_RELWIDTH+XBP_RELHEIGHT+XBP_RELXPOS+XBP_RELYPOS

RETURN self



/*
 * Systemresourcen anfordern
 */
METHOD XbpPreview:create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   LOCAL oFont, i, imax, aFontList

   // XbpStatic ist Parent fr alle weiteren XBPs
   ::xbpStatic:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   // Printer erzeugen
   ::printer:create()

   // XbpDialog wird ohne Titel angezeigt
   ::oView:minButton := .F.
   ::oView:maxButton := .F.
   ::oView:sysMenu   := .F.
   ::oView:titleBar  := .F.
   ::oView:border    := XBPDLG_THINBORDER
   ::oView:create()

   ::oPagePos:create()
   ::oPagePos:setFontCompoundName( "7.Arial" )
   ::oPagePos:paint  := { || ::oPagePos:setCaption( AllTrim(Str(::pageNo())) +;
                             "/" + AllTrim(Str(::numPages())) ) }

   ::oNextPage:create()
   ::oNextPage:setFontCompoundName( "7.Arial" )
   ::oPrevPage:create()
   ::oPrevPage:setFontCompoundName( "7.Arial" )

   // Scrollbars und Fllquadrat zw. Scrollbars erzeugen
   ::oSquare:create()
   ::hScroll:create()
   ::vScroll:create()

   ::hScroll:scroll := {|mp1| ::hScroll( mp1 ) }
   ::vScroll:scroll := {|mp1| ::vScroll( mp1 ) }

   // Gre der XBPs wird in :setSize() eingestellt
   ::setSize( ::currentSize() )

   // Gre der Druckseite abzglich nicht bedruckbarer Rnder berechnen
   // Die Einheit ist 1/10 mm
   aSize       := ::printer:paperSize()
   ::aPageSize := { aSize[5]-aSize[3], aSize[6]-aSize[4] }

   // Presentation Space mit Drucker Device Context verknpfen
   // Der PS bekommt die metrische Einheit 1/10 mm
   ::oPresSpace:create( ::printer, ::aPageSize, GRA_PU_LOMETRIC )

   IF ::oFont == NIL
      // Default Font bereitstellen -> kleiner Nichtproportionalfont
      ::oFont := XbpFont():new( ::oPresSpace )
      ::oFont:familyName := "Courier New"
      ::oFont:nominalPointSize := 8
      ::oFont:generic := .T.
      ::oFont:create()
   ENDIF

   // Presentation Space mit Window Device Context verkn?pfen
   ::oPresSpace:configure( ::oView:drawingArea:winDevice() )

   // Beim Repaint wird alles mit einer weien Box bermalt
   ::aCharAttr := Array( GRA_AS_COUNT )
   ::aAreaAttr := Array( GRA_AA_COUNT )
   ::aAreaAttr [ GRA_AA_COLOR ] := GRA_CLR_WHITE
   ::oPresSpace:setAttrArea( ::aAreaAttr )

   ::oPresSpace:setFont( ::oFont )

   // Beende Applikation, wenn kein Codeblock zugewiesen wurde
   IF ValType( ::drawBlock ) != "B"
      MsgBox( ":drawBlock ist ungltig!" )
      QUIT
   ENDIF

   // Evaluieren des Codeblocks in ":drawBlock" zum Formatieren
   // der ersten Seite. Dieser mu eine Routine aufrufen, die
   // die Anzeige durchfhrt, siehe Funktion "FormatPage()"
  AAdd( ::segments, Eval(::drawBlock, ::oPresSpace, 1, @::nRows) )
  ASize( ::segments, ::numPages() )

   // Viewport initialisieren
   aSize       := ::oView:drawingArea:currentSize()
   ::aViewPort := { 0, 0, aSize[1], aSize[2] }
   ::oView:drawingArea:paint := {|| ::drawPage() }
   ::zoom()

RETURN self



/*
 * Systemresourcen freigeben
 */
METHOD XbpPreview:destroy
   ::xbpStatic:destroy()
   ::aViewPort := ;
   ::aPageSize := ;
   ::oFont     := NIL
RETURN self



/*
 * Gre von XbpStatic und enthaltenen XBPs ndern
 */
METHOD XbpPreview:setSize( aSize )

   ::xbpStatic:setSize( aSize )
   aSize := ::currentSize()

   ::oPagePos:setPos ( { 0, 0 } )
   ::oPagePos:setSize( { 40, 16 } )
   ::oNextPage:setPos ( { 40, 0 } )
   ::oNextPage:setSize( { 20, 16 } )
   ::oPrevPage:setPos ( { 60, 0 } )
   ::oPrevPage:setSize( { 20, 16 } )

   ::hScroll:setPos ( { 80, 0 } )
   ::hScroll:setSize( { aSize[1]-16-80, 16 } )
   ::vScroll:setPos ( { aSize[1]-16, 16 } )
   ::vScroll:setSize( { 16, aSize[2]-16 } )
   ::oSquare:setPos ( { aSize[1]-16,  0 } )
   ::oSquare:setSize( { 16, 16 } )
   ::oView  :setPos ( { 0 , 16 } )
   ::oView  :setSize( { aSize[1]-16, aSize[2]-16 } )

RETURN self



/*
 * Horizontal scrollen
 */
METHOD XbpPreview:hScroll( mp1 )
   LOCAL nScrollPos := mp1[1]
   LOCAL nCommand   := mp1[2]
   LOCAL nScroll

   IF (nScrollPos == 0                               .AND. ;
       ::hScroll:getData() == 0)                     .OR.  ;
      (nScrollPos == ::hScroll:setRange()[2]         .AND. ;
       ::hScroll:getData() == ::hScroll:setRange()[2] )
       RETURN self
   ENDIF

   DO CASE
   CASE nCommand == XBPSB_PREVPOS
      nScroll = ::hScroll:getData() - ::hScroll:setScrollBoxSize()
      ::hScroll:setData( Max(nScroll, 0) )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_NEXTPOS
      nScroll = ::hScroll:getData() + ::hScroll:setScrollBoxSize()
      ::hScroll:setData( Min(nScroll, ::hScroll:setRange()[2]) )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_PREVPAGE
      ::hScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_NEXTPAGE
      ::hScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_ENDTRACK
      ::hScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   ENDCASE

RETURN self



/*
 * Vertikal scrollen
 */
METHOD XbpPreview:vScroll( mp1 )
   LOCAL nScrollPos := mp1[1]
   LOCAL nCommand   := mp1[2]
   LOCAL nScroll

   IF (nScrollPos == 0                               .AND. ;
       ::vScroll:getData() == 0)                     .OR.  ;
      (nScrollPos == ::vScroll:setRange()[2]         .AND. ;
       ::vScroll:getData() == ::vScroll:setRange()[2] )
       RETURN self
   ENDIF

   DO CASE
   CASE nCommand == XBPSB_PREVPOS
      nScroll = ::vScroll:getData() - ::vScroll:setScrollBoxSize()
      ::vScroll:setData( Max(nScroll, 0) )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_NEXTPOS
      nScroll = ::vScroll:getData() + ::vScroll:setScrollBoxSize()
      ::vScroll:setData( Min(nScroll, ::vScroll:setRange()[2]) )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_PREVPAGE
      ::vScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_NEXTPAGE
      ::vScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   CASE nCommand == XBPSB_ENDTRACK
      ::vScroll:setData( nScrollPos )
      ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )
   ENDCASE

RETURN self



/*
 * Ursprung vom Viewport setzen (nach Scrollen)
 */
METHOD XbpPreview:setOrigin( nX, nY )
   LOCAL aSize       := ::oView:drawingArea:currentSize()
   LOCAL nViewWidth  := aSize[1]
   LOCAL nViewHeight := aSize[2]
   LOCAL nZoomWidth  := nViewWidth  * ::zoomFactor
   LOCAL nZoomHeight := nViewHeight * ::zoomFactor

   // X Ursprung ist immer <= 0
   nX := - Abs( nX )

   IF nZoomWidth <= nViewWidth
      nX := 0
   ELSEIF nViewWidth - nX >= nZoomWidth
      nX := nViewWidth - nZoomWidth
   ENDIF

   ::aViewPort[1] := nX
   ::aViewPort[3] := nX + nZoomWidth

   // Y Ursprung ist immer <= 0
   // Es mu die Differenz zwischen Gesamthhe und
   // sichtbarem Bereich bercksichtigt werden
   nY := Abs( nY ) - ( nZoomHeight - nViewHeight )

   IF nY > 0 .OR. nZoomHeight <= nViewHeight
      nY := 0
   ELSEIF nViewHeight - nY >= nZoomHeight
      nY := nViewHeight - nZoomHeight
   ENDIF

   ::aViewPort[2] := nY
   ::aViewPort[4] := nY + nZoomHeight

   ::setViewPort( ::aViewPort )
   ::drawPage()

RETURN self



/*
 * Gre der ScrollBox und Range fr beide Scrollbars berechnen
 */
METHOD XbpPreview:calcScrollBox()
   LOCAL aSize       := ::oView:drawingArea:currentSize()
   LOCAL nViewWidth  := aSize[1]
   LOCAL nViewHeight := aSize[2]
   LOCAL nZoomWidth  := aSize[1] * ::zoomFactor
   LOCAL nZoomHeight := aSize[2] * ::zoomFactor

   // Werte fr horizontalen Scrollbar berechnen
   ::hScroll:setRange( { 0, Int(nZoomWidth - nViewWidth) } )
   IF nZoomWidth > nViewWidth
      ::hScroll:setScrollBoxSize( (nZoomWidth-nViewWidth) * (nViewWidth/nZoomWidth) )
   ELSE
      ::hScroll:setScrollBoxSize( nViewWidth )
   ENDIF

   // Werte fr vertikalen Scrollbar berechnen
   ::vScroll:setRange( { 0, Int(nZoomHeight - nViewHeight) } )
   IF nZoomHeight > nViewHeight
      ::vScroll:setScrollBoxSize( (nZoomHeight-nViewHeight) * (nViewHeight/nZoomHeight) )
   ELSE
      ::vScroll:setScrollBoxSize( nViewHeight )
   ENDIF

RETURN self



/*
 * Viewport zoomen
 */
METHOD XbpPreview:zoom( nZoomFactor )
   LOCAL aSize  := ::oView:drawingArea:currentSize()

   DEFAULT nZoomFactor TO ::zoomFactor

   ::zoomFactor := Max( 1, nZoomFactor )

   // Viewport neu berechnen
   aSize[1] := Int( aSize[1] * nZoomFactor )
   aSize[2] := Int( aSize[2] * nZoomFactor )

   ::aViewPort  := {0,0,aSize[1],aSize[2]}
   ::calcScrollBox()

   // Ursprung vom Viewport neu definieren
   ::setOrigin( ::hScroll:getData(), ::vScroll:getData() )

RETURN self



/*
 * Presentation Space abfragen
 */
METHOD XbpPreview:presSpace
RETURN ::oPresSpace



/*
 * Viewport setzen
 */
METHOD XbpPreview:setViewPort( aViewPort )
   LOCAL aOldViewPort := ::oPresSpace:setViewPort()

   IF Valtype( aViewPort ) == "A"
      ::aViewPort := aViewPort
      ::oPresSpace:setViewPort( aViewPort )
   ENDIF
RETURN aOldViewPort



/*
 * Zeichnen der angegebenen Seite
 */
METHOD XbpPreview:drawPage( nPageNo )

   DEFAULT nPageNo TO ::nPagePos

   IF Len(::segments) == 0
      RETURN self
   ENDIF

   // Zeichnung wird durch Ausgabe einer wei gefllten Box gelscht
   GraBox( ::oPresSpace, {0,0}, ::aPageSize, GRA_FILL )

   IF nPageNo != 0 .AND. nPageNo <= ::numPages()
      IF ::segments[nPageNo] == NIL
         ::segments[nPageNo] := Eval( ::drawBlock,  ;
                                      ::oPresSpace, ;
                                      nPageNo,      ;
                                      @::nRows )
      ENDIF

      IF ::segments[nPageNo] != 0
         GraSegDraw( ::presSpace(), ::segments[nPageNo] )
      ENDIF
   ENDIF

RETURN self



/*
 * Druckausgabe beginnen
 * - Der Presentation Space wird mit dem Print Device Context assoziiert
 */
METHOD XbpPreview:startDoc

   ::oPresSpace:configure( ::printer )

   // Ab jetzt wird die Ausgabe in :oPresSpace auf den Spooler umgeleitet
   ::printer:startDoc()

   // Font und Attribute gehen nach :configure() verloren -> neu setzen
   ::oPresSpace:setFont( ::oFont )
   ::oPresSpace:setAttrArea( ::aAreaAttr )
   ::oPresSpace:setAttrString( ::aCharAttr )

RETURN self



/*
 * Druckausgabe beenden
 * - Der Presentation Space wird wieder mit dem Window Device Context assoziiert
 */
METHOD XbpPreview:endDoc

   // Spooler schlieen
   ::printer:endDoc()

   // PS wieder mit Window Device verknpfen
   ::oPresSpace:configure( ::oView:drawingArea:winDevice() )

   // Font und Attribute gehen nach :configure() verloren -> neu setzen
   ::oPresSpace:setFont( ::oFont )
   ::oPresSpace:setAttrArea( ::aAreaAttr )
   ::oPresSpace:setAttrString( ::aCharAttr )

   // Viewport fr Window setzen
   ::zoom()

RETURN self



/*
 * Ausgabe aller Seiten auf dem Drucker
 */
METHOD XbpPreview:print()
   LOCAL i

   IF Len(::segments) == 0
      RETURN self
   ENDIF

   ::startDoc()

   // Lschen Sie die folgenden Kommentare,
   // um das Drucken der gesamt Datenbank zu
   // erlauben. In diesem Beispiel wird nur die
   // aktuelle Seite gedruckt.
   ::drawPage( ::nPagePos )

   // FOR i:=1 TO ::numPages()
   //   ::drawPage( i )
   //
   //   IF i +1 <= ::numPages()
   //      ::printer:newPage()
   //   ENDIF
   // NEXT

   ::endDoc()

RETURN self



/*
 * Ermitteln der Seitenanzahl
 */
METHOD XbpPreview:numPages()
   LOCAL nPages

   IF ::nRows == 0
      nPages := 1
   ELSE
      nPages := Int( RecCount() / ::nRows ) + ;
                IIF( RecCount() % ::nRows != 0, 1, 0 )
   ENDIF
RETURN nPages



/*
 * Ermitteln der Nummer der akt. Seite
 */
METHOD XbpPreview:pageNo()
RETURN ::nPagePos
/*
 * Anzeigen der nchsten Seite
 */
METHOD XbpPreview:drawNextPage()
   IF ::nPagePos +1 <= ::numPages()
      ::nPagePos++
      ::drawPage()
      ::oPagePos:invalidateRect()
   ENDIF
RETURN self



/*
 * Anzeigen der vorhergehenden Seite
 */
METHOD XbpPreview:drawPrevPage()
   IF ::nPagePos -1 > 0
      ::nPagePos--
      ::drawPage()
      ::oPagePos:invalidateRect()
   ENDIF
RETURN self


PROCEDURE MainResize(aOldSize, aNewSize, oSelf)
LOCAL aChild
LOCAL i,iMax

   IF aOldSize[1] == 0 .AND. aOldSize[2] == 0               // was minimize
      aOldSize := aNewSize                                  // New Size

   ELSEIF aNewSize[1] == 0 .OR. aNewSize[2] == 0            // is now minimized
      RETURN

   ELSEIF aNewSize[1] == aOldSize[1] .AND. aNewSize[2] == aOldSize[2]
      RETURN
   ENDIF                                                    // same Size

   oSelf:lockUpdate(.T.)

   aChild := oSelf:Childlist()
   iMax  := LEN(aChild)
   FOR i := 1 TO iMax
      IF aChild[i]:isDerivedFrom( "XbpPreview" )
         aChild[i]:setsize({(aNewSize[1]/4*3)-20,aNewSize[2]-30})
         aChild[i]:Zoom()
      ENDIF
   NEXT

   oSelf:lockUpdate(.F.)
   oSelf:invalidateRect()

RETURN